/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */
package org.objectweb.perseus.persistence.api;

import org.objectweb.perseus.cache.api.CacheEntry;

import java.util.List;
import java.util.Set;

/**
 * It defines a context used in by the PersistenceManager. A context contains
 * a collection of CacheEntry instances and has a status.
 *
 * @author S.Chassande-Barrioz
 */
public interface WorkingSet {

	/**
	 * The status of a non transactional context which is not already
	 * closed.
	 */
	byte CTX_ACTIVE = 1;


	/**
	 * The status of a cloased context (transactional or not)
	 */
	byte CTX_CLOSED = 32;

	/**
	 * Returns the working set's status.
	 * @return one of the constants of the {@link
	 * org.objectweb.perseus.persistence.api.TransactionalPersistenceManager}
	 * interface.
	 */
	byte getStatus();

	/**
	 * Sets the working set's status.
	 * This setter permits also to intercept the working set life cycle.
	 * @param status one of the constants of the {@link
	 * org.objectweb.perseus.persistence.api.TransactionalPersistenceManager}
	 * interface.
	 * @throws PersistenceException if this is a probleme in the interception of
	 * the working set life cycle.
	 */
	void setStatus(byte status) throws PersistenceException;

	/**
	 * @return the entry attached to the working set weither its identifier
	 */
	State lookup(Object oid);

	/**
	 * Attaches an entry to the working set.
	 * @param state is the state which must be attached to the working set
	 * @param oid is the id of the corresponding object
	 */
	
	void bind(State state, Object oid);

	/**
	 * Dettaches an entry from the working set.
	 * @param oid is the identifier of entry which must be detattached from
	 * the working set
     * @return true if the entry was present in the working set
	 */
	boolean unbind(Object oid);

	/**
	 * Removes all entries of the working set.
	 */
	void clear();

	/**
	 * @return the list of entries attached to the working set.
	 */
	Set entries();

    /**
	 * @return the list of entries attached to the working set
	 */
	Set oids();

	/**
	 * @return the user object linked to the working set
	 */
	Object getUserObject();

	/**
	 * Compare the current working set to the parameter.
	 */
	//boolean equals(Object ws);

	/**
	 * @return a ConnectionHolder instance permiting to acces the data support
	 */
	ConnectionHolder getConnectionHolder();

	boolean getWSRetainValues();

	void setWSRetainValues(boolean val);

	boolean getWSRestoreValues();

	void setWSRestoreValues(boolean val);
}
