/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.persistence.concurrency;

import org.objectweb.perseus.concurrency.optimistic.OptimisticConcurrencyManager;
import org.objectweb.perseus.concurrency.lib.TimeStamp;
import org.objectweb.perseus.concurrency.api.ConcurrencyException;
import org.objectweb.perseus.persistence.api.*;
import org.objectweb.perseus.cache.api.CacheEntry;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class POptimisticConcurrencyManager extends OptimisticConcurrencyManager {


    public final static String STORAGE_MANAGER_BINDING = "storage-manager";
    public final static String STATE_MANAGER_BINDING = "state-manager";

    protected StateManager stateManager;

    protected StorageManager storageManager;

    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public String[] listFc() {
        return new String[] {
            STORAGE_MANAGER_BINDING,
            STATE_MANAGER_BINDING };
    }

    public Object lookupFc(String s) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            return storageManager;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            return stateManager;
        } else {
            return super.lookupFc(s);
        }
    }

    public void bindFc(String s, Object o) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            storageManager = (StorageManager) o;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            stateManager = (StateManager) o;
        } else super.bindFc(s, o);
    }

    public void unbindFc(String s) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            storageManager = null;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            stateManager = null;
        } else {
            super.unbindFc(s);
        }
    }

	protected Object getState(Object ctx,
							  Object resourceId,
							  TimeStamp timeStamp,
							  Object hints,
							  boolean isDirtyBefore,
							  boolean isWrite) throws ConcurrencyException {
		CacheEntry ce = (CacheEntry) hints;
		WorkingSet ws = (WorkingSet) ctx;
		State state = ws.lookup(ce.getCeIdentifier());
		if (state == null) {
			state = stateManager.getReferenceState(ce);
			if (state == null) { //no reference state ==> load it
				state = load(ws, ce);
			}
			if (isWrite) {
				//copy on write
				state = stateManager.createState(state);
			} // else  read case
		} else {
			//There is state in the working set
			if (isWrite) {
				if (!isDirtyBefore) {
					//copy on write
					state = stateManager.createState(state);
				} //else already in write
			} // else read case ==> use the state of the workingset
		}
		return state;
    }

	private State load(WorkingSet ws, CacheEntry ce)
		throws ConcurrencyException {
		State state = stateManager.createState(ce);
		stateManager.setReferenceState(state.getCacheEntry(), state);
		try {
			storageManager.read(ws, ce.getCeIdentifier(), state);
		} catch (PersistenceException e) {
			stateManager.setReferenceState(state.getCacheEntry(), null);
			stateManager.destroyState(state);
			throw new NoDSIConcurrencyException(e);
		}
		stateManager.makeClean(state);
		return state;
	}
}
